/* This program generates input file '*.dat' in the format required for
  'crystal_new.for' which is a modified version of 'crystal.for' 

  The selected crystal structures;
   Structure		Code
    CUBIC		1
    FCC			2
    HEXAGONAL		3
    ORTHORHOMIBIC	4
    TETRAGONAL		5

INPUT :
-----
The format of input data file to this program (see ppt.dat, first line is comment line);

TYPE		CODE 	a 	b 	c 	PRECIPITATE		YES_OR_NO
FCC   		2 	10.6454	0	0	Cr_21.34Fe_1.66C_6	0
CUBIC   	1 	8.9	0	0	Cr_12Fe_36Mo_10		1

INTERMEDIATE DATA FILE:
-----------------------
1 
1 
1 
6 
3 
1 
4.52500 5.08700 6.74300 
0.030 
1.322581 
1.188406 
29.000000 
1 
1.322581 
2.484848 
90.000000 
1 
1.188406 
2.484848 
61.000000 
0
1 
1 
1 
6 
1 
1 
10.59100  
0.030 
1.322581 
1.188406 
29.000000 
1 
1.322581 
2.484848 
90.000000 
1 
1.188406 
2.484848 
61.000000 
0
1 

OUTPUT:
------
	 Precipitates data file : ppt.dat 

	 Camera length : 1370.000 mm 

	 Camera Constant : 0.025000 A 

	 Maximum range of Miller Index: 6

	 Accuracy : 0.030

	 R[0] = 7.50000  d[0] = 4.56667 
	 R[1] = 10.00000  d[1] = 3.42500 	 Angle = 42.500 [R0 - R1]

	 R[2] = 6.75000  d[2] = 5.07407 	 Angle = 90.000 [R0 - R2]


 *********** SELECTED PRECIPITATES **************

TYPE		 CODE  a		b		c		PRECIPITATE 

ORTHORHOMIBIC	4  4.525		5.087		6.743		Fe_3C 
CUBIC		 1  10.591		0		0		Fe_23(C,B)_6 
FCC		 2  11.082		0		0		M_6C 
FCC		 2  10.621		0		0		M_23C_6 
HEXAGONAL	 3  2.888		0		4.559		M_2C_[(Cr,Mo)_2C] 
ORTHORHOMIBIC	4  4.526		7.01		12.142		M_7C_3 
HEXAGONAL	 3  2.99		0		4.72		WC_alpha 
CUBIC		 1  4.248		0		0		WC_beta 
HEXAGONAL	 3  2.986		0		4.712		W_2C 
HEXAGONAL	 3  2.901		0		2.83		WC 
HEXAGONAL	 3  10.58		0		13.35		WC_x 
ORTHORHOMIBIC	4  4.728		6.009		5.193		W_2C_120 
CUBIC		 1  4.126		0		0		W_2N 
           ***********************************************************
           **                                                       **
           Lattice Parameters "a", "b", "c" ?

           ORTHORHOMBIC SYSTEM    4.5250    5.0870    6.7430 ANGSTROMS

D1=   4.56666708
 D2=   3.42499995
 Angle between  4.56666708 and   3.42499995 =   42.5
------------------------------------------------------------------------------
    Vector 1                   Vector 2               Zone Axis     

  (h  k  l)      d        (h  k  l)     d         [U      V      W]    Angle

  1. -1.  0.  3.3810     1.  0.  0.  4.5250     0.0000  0.0000  1.0000  41.6
 D1=   4.56666708
 D2=   5.07407379
 Angle between  4.56666708 and   5.07407379 =   90.
------------------------------------------------------------------------------
    Vector 1                   Vector 2               Zone Axis     

  (h  k  l)      d        (h  k  l)     d         [U      V      W]    Angle

  0. -1.  0.  5.0870     1.  0.  0.  4.5250     0.0000  0.0000  1.0000  90.0
 D1=   3.42499995
 D2=   5.07407379
 Angle between  3.42499995 and   5.07407379 =   47.5
------------------------------------------------------------------------------
    Vector 1                   Vector 2               Zone Axis     

  (h  k  l)      d        (h  k  l)     d         [U      V      W]    Angle

  0. -1.  0.  5.0870     1. -1.  0.  3.3810     0.0000  0.0000  1.0000  48.3
           ***********************************************************
           **                                                       **
---- Sree Harsha Lalam
 */

#include<stdio.h>
#include<string.h>
#include<stdlib.h>

int No_of_lines(char iname[])
{ int no_of_data=0;
  FILE *ipt;
  char s[2000];

ipt=fopen(iname,"r");
do{fscanf(ipt,"%[^\n]\n",s);
if(s != " " || s != "\n") ++no_of_data;
}while(!feof(ipt));
fclose(ipt);

return(no_of_data);
}
void TETRAGONAL(float R[],float A[],float L,float Lambda,int N,int ED,int CConstant,int Cry_Equ,int Max_Mil,float accuracy,char oname[],float a,float b,float c)
{ FILE *opt;
  int i, j;
   float r1,r2, l;

opt=fopen(oname,"a");
 fprintf(opt,"%d \n",ED);
  fprintf(opt,"%d \n",CConstant);
  fprintf(opt,"%d \n",Cry_Equ);
  fprintf(opt,"%d \n",Max_Mil);

/*  Choose Crystal System
 
           1 = Cubic          4 = Hexagonal or Trigonal
           2 = Tetragonal     5 = Monoclinic
           3 = Orthorhombic   6 = Triclinic
                                 */
fprintf(opt,"2 \n");

 /*    Choose Lattice Type
 
              1 = Primitive
              2 = Body Centered (Cubic, Tetragonal, Orthorhombic)
              3 = Face Centered (Cubic, Orthorhombic)
              4 = A-Centered (Orthorhombic)
              5 = B-Centered (Orthorhombic)
              6 = C-Centered (Orthorhombic or Monoclinic)  */
fprintf(opt,"1 \n");

/* Lattice Parameters "a" and "c" ? */

fprintf(opt,"%.5f %.5f \n",a,c);

fprintf(opt,"%.3f \n",accuracy);

for(i=0;i<(N-1);++i)
  { r1=R[i];
     r1=(L*Lambda)/r1;
           for(j=i+1;j<N;++j)
           { r2=R[j];
             r2=(L*Lambda)/r2;
             l=A[i]-A[j];
             if(l < 0 ) l= -l;
             if (l > 90) l=180-l;
             fprintf(opt,"%f \n",r1);
	fprintf(opt,"%f \n",r2);
	fprintf(opt,"%f \n",l);
       if(  i < (N-2)) fprintf(opt,"1 \n");

            }
  }
fprintf(opt,"0\n");
fclose(opt);
}


void ORTHORHOMIBIC(float R[],float A[],float L,float Lambda,int N,int ED,int CConstant,int Cry_Equ,int Max_Mil,float accuracy,char oname[],float a,float b,float c)
{ FILE *opt;
  int i, j;
   float r1,r2, l;

opt=fopen(oname,"a");
 fprintf(opt,"%d \n",ED);
  fprintf(opt,"%d \n",CConstant);
  fprintf(opt,"%d \n",Cry_Equ);
  fprintf(opt,"%d \n",Max_Mil);

/*  Choose Crystal System
 
           1 = Cubic          4 = Hexagonal or Trigonal
           2 = Tetragonal     5 = Monoclinic
           3 = Orthorhombic   6 = Triclinic
                                 */
fprintf(opt,"3 \n");

 /*    Choose Lattice Type
 
              1 = Primitive
              2 = Body Centered (Cubic, Tetragonal, Orthorhombic)
              3 = Face Centered (Cubic, Orthorhombic)
              4 = A-Centered (Orthorhombic)
              5 = B-Centered (Orthorhombic)
              6 = C-Centered (Orthorhombic or Monoclinic)  */
fprintf(opt,"1 \n");

/* Lattice Parameters "a", "b", "c" ? */

fprintf(opt,"%.5f %.5f %.5f \n",a,b,c);

fprintf(opt,"%.3f \n",accuracy);

for(i=0;i<(N-1);++i)
  { r1=R[i];
     r1=(L*Lambda)/r1;
           for(j=i+1;j<N;++j)
           { r2=R[j];
             r2=(L*Lambda)/r2;
             l=A[i]-A[j];
             if(l < 0 ) l= -l;
             if (l > 90) l=180-l;
             fprintf(opt,"%f \n",r1);
	fprintf(opt,"%f \n",r2);
	fprintf(opt,"%f \n",l);
       if(  i < (N-2)) fprintf(opt,"1 \n");

            }
  }
fprintf(opt,"0\n");
fclose(opt);
}




void HEXAGONAL(float R[],float A[],float L,float Lambda,int N,int ED,int CConstant,int Cry_Equ,int Max_Mil,float accuracy,char oname[],float a,float c)
{ FILE *opt;
  int i, j;
   float r1,r2, l;

opt=fopen(oname,"a");
 fprintf(opt,"%d \n",ED);
  fprintf(opt,"%d \n",CConstant);
  fprintf(opt,"%d \n",Cry_Equ);
  fprintf(opt,"%d \n",Max_Mil);

/*  Choose Crystal System
 
           1 = Cubic          4 = Hexagonal or Trigonal
           2 = Tetragonal     5 = Monoclinic
           3 = Orthorhombic   6 = Triclinic
                                 */
fprintf(opt,"4 \n");

 /*    Choose Lattice Type
 
              1 = Primitive
              2 = Body Centered (Cubic, Tetragonal, Orthorhombic)
              3 = Face Centered (Cubic, Orthorhombic)
              4 = A-Centered (Orthorhombic)
              5 = B-Centered (Orthorhombic)
              6 = C-Centered (Orthorhombic or Monoclinic)  */
fprintf(opt,"1 \n");

/* Lattice Parameters "a",  "c" ? */

fprintf(opt,"%.5f %.5f \n",a,c);

fprintf(opt,"%.3f \n",accuracy);

for(i=0;i<(N-1);++i)
  { r1=R[i];
     r1=(L*Lambda)/r1;
           for(j=i+1;j<N;++j)
           { r2=R[j];
             r2=(L*Lambda)/r2;
             l=A[i]-A[j];
             if(l < 0 ) l= -l;
             if (l > 90) l=180-l;
             fprintf(opt,"%f \n",r1);
	fprintf(opt,"%f \n",r2);
	fprintf(opt,"%f \n",l);
       if(  i < (N-2)) fprintf(opt,"1 \n");

            }
  }
fprintf(opt,"0\n");
fclose(opt);
}


void FCC(float R[], float A[],float L,float Lambda,int N,int ED,int CConstant,int Cry_Equ,int Max_Mil,float accuracy,char oname[],float a)
{ FILE *opt;
  int i, j;
   float r1,r2, l;

opt=fopen(oname,"a");
 fprintf(opt,"%d \n",ED);
  fprintf(opt,"%d \n",CConstant);
  fprintf(opt,"%d \n",Cry_Equ);
  fprintf(opt,"%d \n",Max_Mil);

/*  Choose Crystal System
 
           1 = Cubic          4 = Hexagonal or Trigonal
           2 = Tetragonal     5 = Monoclinic
           3 = Orthorhombic   6 = Triclinic
                                 */
fprintf(opt,"1 \n");

 /*    Choose Lattice Type
 
              1 = Primitive
              2 = Body Centered (Cubic, Tetragonal, Orthorhombic)
              3 = Face Centered (Cubic, Orthorhombic)
              4 = A-Centered (Orthorhombic)
              5 = B-Centered (Orthorhombic)
              6 = C-Centered (Orthorhombic or Monoclinic)  */
fprintf(opt,"3 \n");

/* Lattice Parameters "a"? */

fprintf(opt,"%.5f  \n",a);

fprintf(opt,"%.3f \n",accuracy);

for(i=0;i<(N-1);++i)
  { r1=R[i];
     r1=(L*Lambda)/r1;
           for(j=i+1;j<N;++j)
           { r2=R[j];
             r2=(L*Lambda)/r2;
             l=A[i]-A[j];
             if(l < 0 ) l= -l;
             if (l > 90) l=180-l;
             fprintf(opt,"%f \n",r1);
	fprintf(opt,"%f \n",r2);
	fprintf(opt,"%f \n",l);
       if(  i < (N-2)) fprintf(opt,"1 \n");

            }
  }
fprintf(opt,"0\n");
fclose(opt);
}





void CUBIC(float R[],float A[],float L,float Lambda,int N,int ED,int CConstant,int Cry_Equ,int Max_Mil,float accuracy,char oname[],float a)
{ FILE *opt;
  int i, j;
   float r1,r2, l;
  
opt=fopen(oname,"a");

  fprintf(opt,"%d \n",ED);
  fprintf(opt,"%d \n",CConstant);
  fprintf(opt,"%d \n",Cry_Equ);
  fprintf(opt,"%d \n",Max_Mil);

/*  Choose Crystal System
 
           1 = Cubic          4 = Hexagonal or Trigonal
           2 = Tetragonal     5 = Monoclinic
           3 = Orthorhombic   6 = Triclinic
                                 */
fprintf(opt,"1 \n");

 /*    Choose Lattice Type
 
              1 = Primitive
              2 = Body Centered (Cubic, Tetragonal, Orthorhombic)
              3 = Face Centered (Cubic, Orthorhombic)
              4 = A-Centered (Orthorhombic)
              5 = B-Centered (Orthorhombic)
              6 = C-Centered (Orthorhombic or Monoclinic)  */
fprintf(opt,"1 \n");

/* Lattice Parameters "a"? */

fprintf(opt,"%.5f  \n",a);

fprintf(opt,"%.3f \n",accuracy);

for(i=0;i<(N-1);++i)
  { r1=R[i];
     r1=(L*Lambda)/r1;
           for(j=i+1;j<N;++j)
           { r2=R[j];
             r2=(L*Lambda)/r2;
             l=A[i]-A[j];
             if(l < 0 ) l= -l;
             if (l > 90) l=180-l;
             fprintf(opt,"%f \n",r1);
	fprintf(opt,"%f \n",r2);
	fprintf(opt,"%f \n",l);
       if(  i < (N-2)) fprintf(opt,"1 \n");

            }
  }
fprintf(opt,"0\n");
fclose(opt);
}

main()
{FILE *opt,*ipt,*rpt;
char S[2000];
 char rname[60],name[60],oname[60],iname[80],type[80],scode[60],sa[60],sb[60],sc[60],scheck[90],PRECIPITATE[80];
 float L,Lambda,R[20],A[20],d,a,b,c,accuracy;
  int check,N,i,ED,CConstant, Cry_Equ,Max_Mil,code,N_of_lines=0;

printf("\n FILE name to input to CRYSTAL.FOR : ");
scanf("%s",name);
strcpy(oname,name);
strcpy(rname,name);

strcat(oname,".dat");
strcat(rname,".out");

if((rpt=fopen(rname,"r")) != NULL)
  { printf("\n \t CAUTION : File '%s' already exists. \n \t\t Do you want me to OVERWRITE on it ? (0/1)",rname);
     scanf("%d",&check);
    if(check != 1)
      { fclose(rpt);
        printf("\n\n\t\t Rerun the program ... \n\n");
         exit(0);
       }
   fclose(rpt);
   }
rpt=fopen(rname,"w");

opt=fopen(oname,"w");
fclose(opt);

printf("\n INPUT FILE name of PRECIPITATES DATA : ");
scanf("%s",iname);


if((ipt=fopen(iname,"r")) == NULL)
{ printf("\n\n\t\t FILE '%s' NOT found .... \n",iname);
  exit(0);
 }
fprintf(rpt,"\n\n\t Precipitates data file : %s ",iname);

printf("\n Camera Length (mm) : ");
 scanf("%f",&L); 
 
fprintf(rpt,"\n\n\t Camera length : %.3f mm ",L);

printf("\n \t ********************************************* \n");
printf("\n \t \t FOR  JEOL 200 at 200 kV on 2-6-2000 \n");
printf("\t \t \t  Camera Length[cm] \t Camera Constant  \n");
printf("\t \t \t     82             \t     0.02439 \n");
printf("\t \t \t     137            \t     0.02452 \n");
printf("\t \t \t     205            \t     0.02527 \n");
printf("\n \t ********************************************* \n");


printf("\n Camera Constant (A) : ");
 scanf("%f",&Lambda);


fprintf(rpt,"\n\n\t Camera Constant : %f A ",Lambda);

/* Select the ED */
ED=1;

/* Is the Camera Constant known ? */
CConstant=1; 

printf("\n Do you wish to avoid crystallographically \n\t\t equivalent solutions (0 = No, 1 = Yes) : ");

 scanf("%d",&Cry_Equ);

  fprintf(opt,"1 \n");

   printf("\n Specify maximum range of Miller Index \n\t\t\t  Typical value = 3  (avoid values > 4)\n\t\t\t Larger values lead to longer run times :");
 scanf("%d",&Max_Mil);

fprintf(rpt,"\n\n\t Maximum range of Miller Index: %d",Max_Mil);

 printf("\n Accuracy of measurement ? (Typical value 0.03) :");
 scanf("%f",&accuracy);

fprintf(rpt,"\n\n\t Accuracy : %.3f\n",accuracy);

printf("\n No. of Planes to Index: ");
 scanf("%d",&N);
A[0]=0;

for(i=0;i<N;++i)
 { printf( "\n R%d (mm): ",i);
   scanf("%f",&R[i]);

d=(L*Lambda)/R[i];
 fprintf(rpt,"\n\t R[%d] = %.5f  d[%d] = %.5f ",i,R[i],i,d);
   if(i != 0)
   {printf("\t Angle between R0 and R%d : ",i);
    scanf("%f",&A[i]);
fprintf(rpt,"\t Angle = %.3f [R0 - R%d]\n",A[i],i);

   }
  }

fprintf(rpt,"\n\n *********** SELECTED PRECIPITATES **************\n\n");

N_of_lines=No_of_lines(iname);

printf("\n\t Number of  PRECIPITATES  in file '%s'  = %d \n",iname,(N_of_lines-1));
fscanf(ipt,"%s %s %s %s %s %s %s",type,scode,sa,sb,sc,PRECIPITATE,scheck);
fprintf(rpt,"%s\t\t %s  %s \t%s \t%s \t%s \n\n",type,scode,sa,sb,sc,PRECIPITATE);

for(i=1;i<N_of_lines;++i)
{
fscanf(ipt,"%s %s %s %s %s %s %s",type,scode,sa,sb,sc,PRECIPITATE,scheck);
/* printf("%s %s %s %s %s %s %s \n",type,scode,sa,sb,sc,PRECIPITATE,scheck); */

code=atoi(scode); 
a=atof(sa);
b=atof(sb); 
c=atof(sc); 
check=atof(scheck);

if(check != 0){

switch(code)
{case 1 :
       fprintf(rpt,"%s\t\t %s  %s \t%s \t%s \t%s \n",type,scode,sa,sb,sc,PRECIPITATE); 
	printf("\n\n Generating data for CUBIC ... a= %.4f [%s]",a,PRECIPITATE);
	CUBIC(R, A,L,Lambda,N,ED,CConstant,Cry_Equ,Max_Mil,accuracy,oname,a);
	break;
case 2 :
        fprintf(rpt,"%s\t\t %s  %s \t%s \t%s \t%s \n",type,scode,sa,sb,sc,PRECIPITATE); 
        printf("\n\n Generating data for FCC ... a= %.4f [%s]",a,PRECIPITATE);
	FCC(R,A,L,Lambda,N,ED,CConstant,Cry_Equ,Max_Mil,accuracy,oname,a);
	break;
case 3 :
        fprintf(rpt,"%s\t %s  %s \t%s \t%s \t%s \n",type,scode,sa,sb,sc,PRECIPITATE); 
        printf("\n\n Generating data for HEXAGONAL ... a= %.4f, c= %.4f [%s]",a,c,PRECIPITATE);
	HEXAGONAL(R,A,L,Lambda,N,ED,CConstant,Cry_Equ,Max_Mil,accuracy,oname,a,c); 
	break;
case 4 :
        fprintf(rpt,"%s\t%s  %s \t%s \t%s \t%s \n",type,scode,sa,sb,sc,PRECIPITATE); 
         printf("\n\n Generating data for ORTHORHOMIBIC... a= %.4f, b= %.4f, c= %.4f [%s]",a,b,c,PRECIPITATE);
	 ORTHORHOMIBIC(R,A,L,Lambda,N,ED,CConstant,Cry_Equ,Max_Mil,accuracy,oname,a,b,c);
	break;
case 5 :
       fprintf(rpt,"%s\t %s  %s \t%s \t%s \t%s \n",type,scode,sa,sb,sc,PRECIPITATE); 
        printf("\n\n Generating data for TETRAGONAL... a= %.4f, b= %.4f, c= %.4f [%s]",a,b,c,PRECIPITATE);
	 TETRAGONAL(R,A,L,Lambda,N,ED,CConstant,Cry_Equ,Max_Mil,accuracy,oname,a,b,c);
	break;
default:
     printf("\n\n\t Sorry! you selected wrong code '%d' in line number '%d' of input data file '%s'\n\n",code,i+1,iname);
      fclose(ipt);
	exit(0);     
}
}
}

opt=fopen(oname,"a");
fprintf(opt,"11 \n");
fclose(opt);
fclose(ipt);
fclose(rpt);

sprintf(S,"crystal_new.ex < %s >> %s",oname,rname);
printf("\n\n\t     USING '%s' \n",S);
system(S);
printf("\n\n\t\t ----- CHECK %s \n\n",rname);
}

